`timescale 1ns / 1ps
`define CLK_PERIOD 20

module ad7606_ddr3_uart_tb();
    reg        clk;
    reg        reset_n;
   
    reg        uart_send_en;
    reg [7:0]  uart_tx_data;
    wire       uart_tx_done;
    
    //LED
    wire [1:0] led;
    wire       uart_tx_cmd;
    //UART
    reg        uart_rx;
    wire       uart_tx;    
    //AD7606
    wire        ad7606_cs_n_o;         
    wire        ad7606_rd_n_o;         
	reg         ad7606_busy_i;         
	reg [15:0]  ad7606_db_i;           
	wire [2:0]  ad7606_os_o;           
	wire        ad7606_reset_o;        
	wire        ad7606_convst_o; 
    //DDR3 Interface
    // Inouts
    wire  [15:0]ddr3_dq;
    wire  [1:0] ddr3_dqs_n;
    wire  [1:0] ddr3_dqs_p;
    // Outputs
    wire [13:0]ddr3_addr;
    wire [2:0] ddr3_ba;
    wire       ddr3_ras_n;
    wire       ddr3_cas_n;
    wire       ddr3_we_n;
    wire       ddr3_reset_n;
    wire [0:0] ddr3_ck_p;
    wire [0:0] ddr3_ck_n;
    wire [0:0] ddr3_cke;
    wire [0:0] ddr3_cs_n;
    wire [1:0] ddr3_dm;
    wire [0:0] ddr3_odt;
    
    initial clk= 1;
    always#(`CLK_PERIOD/2) clk = ~clk;

    initial begin
      reset_n=1'b0;
      ad7606_db_i = 16'h1234;
      ad7606_busy_i = 0; 
      #(`CLK_PERIOD*10);
      reset_n=1'b1;
      uart_send_en = 1'b0;
      uart_tx_data = 8'd0;
      @(posedge ad7606_ddr3_uart.ddr3_init_done);//ȴDDR3ʼ
      #(`CLK_PERIOD*10+1);
      //ɼ,洮ڷ55 A5 00 00 00 00 00 F0
      tx_data(8'h55);
      #(`CLK_PERIOD*10+1);
    
      tx_data(8'hA5);
      #(`CLK_PERIOD*10+1);
    
      tx_data(8'h00);
      #(`CLK_PERIOD*10+1);
    
      tx_data(8'h00);
      #(`CLK_PERIOD*10+1);
    
      tx_data(8'h00);
      #(`CLK_PERIOD*10+1);
    
      tx_data(8'h00);
      #(`CLK_PERIOD*10+1);
    
      tx_data(8'h00);
      #(`CLK_PERIOD*10+1);
    
      tx_data(8'hF0);
      
      repeat(512) begin
        @(posedge ad7606_ddr3_uart.uart_tx_done);
      end
      
      #2000;
      
      $stop;
      
    end
    
    task tx_data;
        input [7:0] data_begin;
        begin
            uart_send_en = 1;
            uart_tx_data = data_begin;
            #(`CLK_PERIOD);
            uart_send_en = 1'b0;
            @(posedge uart_tx_done);
        end
    endtask


    uart_byte_tx uart_byte_tx_cmd(
      .clk       (clk          ),//50Mʱ,
      .reset     (~reset_n     ),//ģ鸴λ,
      .data_byte (uart_tx_data ),//8bit,
      .send_en   (uart_send_en ),//ʹ,
      .baud_set  (4'd4         ),//ڲ
      .uart_tx   (uart_tx_cmd  ),//Rs232ź,
      .tx_done   (uart_tx_done ),//һηɱ־,
      .uart_state(             ) //״̬
    );

    
    always@(posedge ad7606_convst_o)begin
        #40;
        ad7606_busy_i = 1;
        #3960;
        ad7606_busy_i = 0;    
    end
        
    always@(negedge ad7606_rd_n_o)begin
        ad7606_db_i <= ad7606_db_i + 1;
    end
    
    ad7606_ddr3_uart ad7606_ddr3_uart(
       .clk(clk),    //ʱź
       .reset_n(reset_n),  //λź
        //LED
       .led(led),
       //UART
       .uart_rx(uart_tx_cmd),
       .uart_tx(uart_tx),
       
       //AD7606
       .ad7606_cs_n_o(ad7606_cs_n_o), 
       .ad7606_rd_n_o(ad7606_rd_n_o), 
       .ad7606_busy_i(ad7606_busy_i), 
       .ad7606_db_i(ad7606_db_i),   
       .ad7606_os_o(ad7606_os_o),   
       .ad7606_reset_o(ad7606_reset_o),
       .ad7606_convst_o(ad7606_convst_o),
        
        //DDR
        // Inouts
       .ddr3_dq(ddr3_dq),
       .ddr3_dqs_n(ddr3_dqs_n),
       .ddr3_dqs_p(ddr3_dqs_p),
       // Outputs
       .ddr3_addr(ddr3_addr),
       .ddr3_ba(ddr3_ba),
       .ddr3_ras_n(ddr3_ras_n),
       .ddr3_cas_n(ddr3_cas_n),
       .ddr3_we_n(ddr3_we_n),
       .ddr3_reset_n(ddr3_reset_n),
       .ddr3_ck_p(ddr3_ck_p),
       .ddr3_ck_n(ddr3_ck_n),
       .ddr3_cke(ddr3_cke),
       .ddr3_cs_n(ddr3_cs_n),
       .ddr3_dm(ddr3_dm),
       .ddr3_odt(ddr3_odt)	    
    );
    
     ddr3_model ddr3_model(
          .rst_n  (ddr3_reset_n),
          .ck     (ddr3_ck_p   ),
          .ck_n   (ddr3_ck_n   ),
          .cke    (ddr3_cke    ),
          .cs_n   (ddr3_cs_n   ),
          .ras_n  (ddr3_ras_n  ),
          .cas_n  (ddr3_cas_n  ),
          .we_n   (ddr3_we_n   ),
          .dm_tdqs(ddr3_dm     ),
          .ba     (ddr3_ba     ),
          .addr   (ddr3_addr   ),
          .dq     (ddr3_dq     ),
          .dqs    (ddr3_dqs_p  ),
          .dqs_n  (ddr3_dqs_n  ),
          .tdqs_n (            ),
          .odt    (ddr3_odt    )
    ); 
          
endmodule
